unit Box2dweb;

interface

//Box2D.Common

type
  Jb2Color = class external 'b2Color'
    r: Float;
    g: Float;
    b: Float;
    color: Float;
    constructor Create(rr: Float; gg: Float; bb: Float);
    procedure Set(rr: Float; gg: Float; bb: Float);
  end;

//Box2D.Common

type
  Jb2Settings = class external 'b2Settings'
    procedure b2Assert(a: Boolean);
    function b2MixFriction(friction1: Float; friction2: Float): Float;
    function b2MixRestitution(restitution1: Float; restitution2: Float): Float;
    b2_aabbExtension: Float;
    b2_aabbMultiplier: Float;
    b2_angularSleepTolerance: Float;
    b2_angularSlop: Float;
    b2_contactBaumgarte: Float;
    b2_linearSleepTolerance: Float;
    b2_linearSlop: Float;
    b2_maxAngularCorrection: Float;
    b2_maxLinearCorrection: Float;
    b2_maxManifoldPoints: Float;
    b2_maxRotation: Float;
    b2_maxRotationSquared: Float;
    b2_maxTOIContactsPerIsland: Float;
    b2_maxTOIJointsPerIsland: Float;
    b2_maxTranslation: Float;
    b2_maxTranslationSquared: Float;
    b2_pi: Float;
    b2_polygonRadius: Float;
    b2_timeToSleep: Float;
    b2_toiSlop: Float;
    b2_velocityThreshold: Float;
    USHRT_MAX: Float;
    VERSION: String;
  end;

//Box2D.Common.Math

type
  Jb2Mat22 = class external 'b2Mat22'
    col1: Jb2Vec2;
    col2: Jb2Vec2;
    constructor Create;
    procedure Abs;
    procedure AddM(m: Jb2Mat22);
    function Copy: Jb2Mat22;
    function FromAngle(angle: Float): Jb2Mat22;
    function FromVV(c1: Jb2Vec2; c2: Jb2Vec2): Jb2Mat22;
    function GetAngle: Float;
    function GetInverse(out: Jb2Mat22): Jb2Mat22;
    procedure Set(angle: Float);
    procedure SetIdentity;
    procedure SetM(m: Jb2Mat22);
    procedure SetVV(c1: Jb2Vec2; c2: Jb2Vec2);
    procedure SetZero;
    function Solve(out: Jb2Vec2; bX: Float; bY: Float): Jb2Vec2;
  end;

//Box2D.Common.Math

type
  Jb2Mat33 = class external 'b2Mat33'
    col1: Jb2Vec3;
    col2: Jb2Vec3;
    col3: Jb2Vec3;
    constructor Create(c1: Jb2Vec3; c2: Jb2Vec3; c3: Jb2Vec3);
    procedure AddM(m: Jb2Mat33);
    function Copy: Jb2Mat33;
    procedure SetIdentity;
    procedure SetM(m: Jb2Mat33);
    procedure SetVVV(c1: Jb2Vec3; c2: Jb2Vec3; c3: Jb2Vec3);
    procedure SetZero;
    function Solve22(out: Jb2Vec2; bX: Float; bY: Float): Jb2Vec2;
    function Solve33(out: Jb2Vec3; bX: Float; bY: Float; bZ: Float): Jb2Vec3;
  end;

//Box2D.Common.Math

type
  Jb2Math = class external 'b2Math'
    function IsValid(x: Float): Boolean;
    function Dot(a: Jb2Vec2; b: Jb2Vec2): Float;
    function CrossVV(a: Jb2Vec2; b: Jb2Vec2): Float;
    function CrossVF(a: Jb2Vec2; s: Float): Jb2Vec2;
    function CrossFV(s: Float; a: Jb2Vec2): Jb2Vec2;
    function MulMV(A: Jb2Mat22; v: Jb2Vec2): Jb2Vec2;
    function MulTMV(A: Jb2Mat22; v: Jb2Vec2): Jb2Vec2;
    function MulX(T: Jb2Transform; v: Jb2Vec2): Jb2Vec2;
    function MulXT(T: Jb2Transform; v: Jb2Vec2): Jb2Vec2;
    function AddVV(a: Jb2Vec2; b: Jb2Vec2): Jb2Vec2;
    function SubtractVV(a: Jb2Vec2; b: Jb2Vec2): Jb2Vec2;
    function Distance(a: Jb2Vec2; b: Jb2Vec2): Float;
    function DistanceSquared(a: Jb2Vec2; b: Jb2Vec2): Float;
    function MulFV(s: Float; a: Jb2Vec2): Jb2Vec2;
    function AddMM(A: Jb2Mat22; B: Jb2Mat22): Jb2Mat22;
    function MulMM(A: Jb2Mat22; B: Jb2Mat22): Jb2Mat22;
    function MulTMM(A: Jb2Mat22; B: Jb2Mat22): Jb2Mat22;
    function Abs(a: Float): Float;
    function AbsV(a: Jb2Vec2): Jb2Vec2;
    function AbsM(A: Jb2Mat22): Jb2Mat22;
    function Min(a: Float; b: Float): Float;
    function MinV(a: Jb2Vec2; b: Jb2Vec2): Jb2Vec2;
    function Max(a: Float; b: Float): Float;
    function MaxV(a: Jb2Vec2; b: Jb2Vec2): Jb2Vec2;
    function Clamp(a: Float; low: Float; high: Float): Float;
    function ClampV(a: Jb2Vec2; low: Jb2Vec2; high: Jb2Vec2): Jb2Vec2;
    procedure Swap(a: Variant; b: Variant);
    function Random: Float;
    function RandomRange(lo: Float; hi: Float): Float;
    function NextPowerOfTwo(x: Float): Float;
    function IsPowerOfTwo(x: Float): Boolean;
    b2Vec2_zero: Jb2Vec2;
    b2Mat22_identity: Jb2Mat22;
    b2Transform_identity: Jb2Transform;
  end;

//Box2D.Common.Math

type
  Jb2Sweep = class external 'b2Sweep'
    a: Float;
    a0: Float;
    c: Jb2Vec2;
    c0: Jb2Vec2;
    localCenter: Jb2Vec2;
    t0: Jb2Vec2;
    procedure Advance(t: Float);
    function Copy: Jb2Sweep;
    procedure GetTransform(xf: Jb2Transform; alpha: Float);
    procedure Set(other: Jb2Sweep);
  end;

//Box2D.Common.Math

type
  Jb2Transform = class external 'b2Transform'
    position: Jb2Vec2;
    R: Jb2Mat22;
    constructor Create(pos: Jb2Vec2; r: Jb2Mat22);
    function GetAngle: Float;
    procedure Initialize(pos: Jb2Vec2; r: Jb2Mat22);
    procedure Set(x: Jb2Transform);
    procedure SetIdentity;
  end;

//Box2D.Common.Math

type
  Jb2Vec2 = class external 'b2Vec2'
    x: Float;
    y: Float;
    constructor Create(x: Float; y: Float);
    procedure Abs;
    procedure Add(v: Jb2Vec2);
    function Copy: Jb2Vec2;
    procedure CrossFV(s: Float);
    procedure CrossVF(s: Float);
    function GetNegative: Jb2Vec2;
    function IsValid: Boolean;
    function Length: Float;
    function LengthSquared: Float;
    function Make(x: Float; y: Float): Jb2Vec2;
    procedure MaxV(b: Jb2Vec2);
    procedure MinV(b: Jb2Vec2);
    procedure MulM(A: Jb2Mat22);
    procedure Multiply(a: Float);
    procedure MulTM(A: Jb2Mat22);
    procedure NegativeSelf;
    function Normalize: Float;
    procedure Set; overload;
    procedure Set(x: Float); overload;
    procedure Set(x: Float; y: Float); overload;
    procedure SetV(v: Jb2Vec2);
    procedure SetZero;
    procedure Subtract(v: Jb2Vec2);
  end;

//Box2D.Common.Math

type
  Jb2Vec3 = class external 'b2Vec3'
    x: Float;
    y: Float;
    z: Float;
    constructor Create(x: Float; y: Float; z: Float);
    procedure Add(v: Jb2Vec3);
    function Copy: Jb2Vec3;
    function GetNegative: Jb2Vec3;
    procedure Multiply(a: Float);
    procedure NegativeSelf;
    procedure Set; overload;
    procedure Set(x: Float); overload;
    procedure Set(x: Float; y: Float); overload;
    procedure Set(x: Float; y: Float; z: Float); overload;
    procedure SetV(v: Jb2Vec3);
    procedure SetZero;
    procedure Subtract(v: Jb2Vec3);
  end;

//Box2D.Collision

type
  Jb2AABB = class external 'b2AABB'
    lowerBound: JBox2D.Common.Math.b2Vec2;
    upperBound: JBox2D.Common.Math.b2Vec2;
    function Combine(aabb1: Jb2AABB; aabb2: Jb2AABB): Jb2AABB;
    procedure Combine(aabb1: Jb2AABB; aabb2: Jb2AABB);
    function Contains(aabb: Jb2AABB): Boolean;
    function GetCenter: JBox2D.Common.Math.b2Vec2;
    function GetExtents: JBox2D.Common.Math.b2Vec2;
    function IsValid: Boolean;
    function RayCast(output: Jb2RayCastOutput; input: Jb2RayCastInput): Boolean;
    function TestOverlap(other: Jb2AABB): Boolean;
  end;

//Box2D.Collision

type
  Jb2ContactID = class external 'b2ContactID'
    features: JFeatures;
    Key: Float;
    constructor Create;
    function Copy: Jb2ContactID;
    procedure Set(id: Jb2ContactID);
  end;

//Box2D.Collision

type
  Jb2ContactPoint = class external 'b2ContactPoint'
    friction: Float;
    id: Jb2ContactID;
    normal: JBox2D.Common.Math.b2Vec2;
    position: JBox2D.Common.Math.b2Vec2;
    restitution: Float;
    separation: Float;
    shape1: JShapes.b2Shape;
    shape2: JShapes.b2Shape;
    velocity: JBox2D.Common.Math.b2Vec2;
  end;

//Box2D.Collision

type
  Jb2DistanceInput = class external 'b2DistanceInput'
    proxyA: Jb2DistanceProxy;
    proxyB: Jb2DistanceProxy;
    transformA: JBox2D.Common.Math.b2Transform;
    transformB: JBox2D.Common.Math.b2Transform;
    useRadii: Boolean;
  end;

//Box2D.Collision

type
  Jb2DistanceOutput = class external 'b2DistanceOutput'
    distance: Float;
    iterations: Float;
    pointA: JBox2D.Common.Math.b2Vec2;
    pointB: JBox2D.Common.Math.b2Vec2;
  end;

//Box2D.Collision

type
  Jb2DistanceProxy = class external 'b2DistanceProxy'
    m_count: Float;
    m_radius: Float;
    m_vertices: array of JBox2D.Common.Math.b2Vec2;
    function GetSupport(d: JBox2D.Common.Math.b2Vec2): Float;
    function GetSupportVertex(d: JBox2D.Common.Math.b2Vec2): JBox2D.Common.Math.b2Vec2;
    function GetVertex(index: Float): JBox2D.Common.Math.b2Vec2;
    function GetVertexCount: Float;
    procedure Set(shape: JShapes.b2Shape);
  end;

//Box2D.Collision

type
  Jb2DynamicTree = class external 'b2DynamicTree'
    constructor Create;
    function CreateProxy(aabb: Jb2AABB; userData: Variant): Jb2DynamicTreeNode;
    procedure DestroyProxy(proxy: Jb2DynamicTreeNode);
    function GetFatAABB(proxy: Jb2DynamicTreeNode): Jb2AABB;
    function GetUserData(proxy: Jb2DynamicTreeNode): Variant;
    function MoveProxy(proxy: Jb2DynamicTreeNode; aabb: Jb2AABB; displacement: JBox2D.Common.Math.b2Vec2): Boolean;
    procedure Query(callback: (proxy: Jb2DynamicTreeNode): Boolean; aabb: Jb2AABB);
    procedure RayCast(callback: (input: Jb2RayCastInput; proxy: Jb2DynamicTreeNode): Float; input: Jb2RayCastInput);
    procedure Rebalance(iterations: Float);
  end;

//Box2D.Collision

type
  Jb2DynamicTreeBroadPhase = class external 'b2DynamicTreeBroadPhase'
    constructor Create;
    function CreateProxy(aabb: Jb2AABB; userData: Variant): Jb2DynamicTreeNode;
    procedure DestroyProxy(proxy: Jb2DynamicTreeNode);
    function GetFatAABB(proxy: Jb2DynamicTreeNode): Jb2AABB;
    function GetProxyCount: Float;
    function GetUserData(proxy: Jb2DynamicTreeNode): Variant;
    procedure MoveProxy(proxy: Jb2DynamicTreeNode; aabb: Jb2AABB; displacement: JBox2D.Common.Math.b2Vec2);
    procedure Query(callback: (proxy: Jb2DynamicTreeNode): Boolean; aabb: Jb2AABB);
    procedure RayCast(callback: (input: Jb2RayCastInput; proxy: Jb2DynamicTreeNode): Float; input: Jb2RayCastInput);
    procedure Rebalance(iterations: Float);
    function TestOverlap(proxyA: Jb2DynamicTreeNode; proxyB: Jb2DynamicTreeNode): Boolean;
    procedure UpdatePairs(callback: (userDataA: Variant; userDataB: Variant));
    procedure Validate;
  end;

//Box2D.Collision

type
  Jb2DynamicTreeNode = class external 'b2DynamicTreeNode'
  end;

//Box2D.Collision

type
  Jb2Manifold = class external 'b2Manifold'
    e_circles: Float;
    e_faceA: Float;
    e_faceB: Float;
    m_localPlaneNormal: JBox2D.Common.Math.b2Vec2;
    m_localPoint: JBox2D.Common.Math.b2Vec2;
    m_pointCount: Float;
    m_points: array of Jb2ManifoldPoint;
    m_type: Float;
    constructor Create;
    function Copy: Jb2Manifold;
    procedure Reset;
    procedure Set(m: Jb2Manifold);
  end;

//Box2D.Collision

type
  Jb2ManifoldPoint = class external 'b2ManifoldPoint'
    m_id: Jb2ContactID;
    m_localpoint: JBox2D.Common.Math.b2Vec2;
    m_normalImpulse: Float;
    m_tangentImpulse: Float;
    constructor Create;
    procedure Reset;
    procedure Set(m: Jb2ManifoldPoint);
  end;

//Box2D.Collision

type
  Jb2OBB = class external 'b2OBB'
    center: JBox2D.Common.Math.b2Vec2;
    extents: JBox2D.Common.Math.b2Vec2;
    R: JBox2D.Common.Math.b2Mat22;
  end;

//Box2D.Collision

type
  Jb2RayCastInput = class external 'b2RayCastInput'
    maxFraction: Float;
    p1: JBox2D.Common.Math.b2Vec2;
    p2: JBox2D.Common.Math.b2Vec2;
    constructor Create(p1: JBox2D.Common.Math.b2Vec2; p2: JBox2D.Common.Math.b2Vec2; maxFraction: Float);
  end;

//Box2D.Collision

type
  Jb2RayCastOutput = class external 'b2RayCastOutput'
    fraction: Float;
    normal: JBox2D.Common.Math.b2Vec2;
  end;

//Box2D.Collision

type
  Jb2Segment = class external 'b2Segment'
    p1: JBox2D.Common.Math.b2Vec2;
    p2: JBox2D.Common.Math.b2Vec2;
    procedure Extend(aabb: Jb2AABB);
    procedure ExtendBackward(aabb: Jb2AABB);
    procedure ExtendForward(aabb: Jb2AABB);
    function TestSegment(lambda: array of Float; normal: JBox2D.Common.Math.b2Vec2; segment: Jb2Segment; maxLambda: Float): Boolean;
  end;

//Box2D.Collision

type
  Jb2SimplexCache = class external 'b2SimplexCache'
    count: Float;
    indexA: array of Float;
    indexB: array of Float;
    metric: Float;
  end;

//Box2D.Collision

type
  Jb2TOIInput = class external 'b2TOIInput'
    proxyA: Jb2DistanceProxy;
    proxyB: Jb2DistanceProxy;
    sweepA: JBox2D.Common.Math.b2Sweep;
    sweepB: JBox2D.Common.Math.b2Sweep;
    tolerance: Float;
  end;

//Box2D.Collision

type
  Jb2WorldManifold = class external 'b2WorldManifold'
    m_normal: JBox2D.Common.Math.b2Vec2;
    m_points: array of JBox2D.Common.Math.b2Vec2;
    constructor Create;
    procedure Initialize(manifold: Jb2Manifold; xfA: JBox2D.Common.Math.b2Transform; radiusA: Float; xfB: JBox2D.Common.Math.b2Transform; radiusB: Float);
  end;

//Box2D.Collision

type
  JFeatures = class external 'Features'
    flip: Float;
    incidentEdge: Float;
    incidentVertex: Float;
    referenceEdge: Float;
  end;

//Box2D.Collision

type
  JIBroadPhase = class external
    function CreateProxy(aabb: Jb2AABB; userData: Variant): Jb2DynamicTreeNode;
    procedure DestroyProxy(proxy: Jb2DynamicTreeNode);
    function GetFatAABB(proxy: Jb2DynamicTreeNode): Jb2AABB;
    function GetProxyCount: Float;
    function GetUserData(proxy: Jb2DynamicTreeNode): Variant;
    procedure MoveProxy(proxy: Jb2DynamicTreeNode; aabb: Jb2AABB; displacement: JBox2D.Common.Math.b2Vec2);
    procedure Query(callback: (proxy: Jb2DynamicTreeNode): Boolean; aabb: Jb2AABB);
    procedure RayCast(callback: (input: Jb2RayCastInput; proxy: Jb2DynamicTreeNode): Float; input: Jb2RayCastInput);
    procedure Rebalance(iterations: Float);
  end;

//Box2D.Collision.Shapes

type
  Jb2CircleShape = class external 'b2CircleShape'(Jb2Shape)
    constructor Create(radius: Float);
    procedure ComputeAABB(aabb: Jb2AABB; xf: JBox2D.Common.Math.b2Transform);
    procedure ComputeMass(massData: Jb2MassData; density: Float);
    function ComputeSubmergedArea(normal: JBox2D.Common.Math.b2Vec2; offset: Float; xf: JBox2D.Common.Math.b2Transform; c: JBox2D.Common.Math.b2Vec2): Float;
    function Copy: Jb2CircleShape;
    function GetLocalPosition: JBox2D.Common.Math.b2Vec2;
    function GetRadius: Float;
    function RayCast(output: Jb2RayCastOutput; input: Jb2RayCastInput; transform: JBox2D.Common.Math.b2Transform): Boolean;
    procedure Set(other: Jb2CircleShape);
    procedure SetLocalPosition(position: JBox2D.Common.Math.b2Vec2);
    procedure SetRadius(radius: Float);
    function TestPoint(xf: JBox2D.Common.Math.b2Transform; p: JBox2D.Common.Math.b2Vec2): Boolean;
  end;

//Box2D.Collision.Shapes

type
  Jb2EdgeChainDef = class external 'b2EdgeChainDef'
    isALoop: Boolean;
    vertexCount: Float;
    vertices: JBox2D.Common.Math.b2Vec2;
    constructor Create;
  end;

//Box2D.Collision.Shapes

type
  Jb2EdgeShape = class external 'b2EdgeShape'(Jb2Shape)
    constructor Create(v1: JBox2D.Common.Math.b2Vec2; v2: JBox2D.Common.Math.b2Vec2);
    procedure ComputeAABB(aabb: Jb2AABB; xf: JBox2D.Common.Math.b2Transform);
    procedure ComputeMass(massData: Jb2MassData; density: Float);
    function ComputeSubmergedArea(normal: JBox2D.Common.Math.b2Vec2; offset: Float; xf: JBox2D.Common.Math.b2Transform; c: JBox2D.Common.Math.b2Vec2): Float;
    function GetLength: Float;
    function GetVertex1: JBox2D.Common.Math.b2Vec2;
    function GetVertex2: JBox2D.Common.Math.b2Vec2;
    function GetCoreVertex1: JBox2D.Common.Math.b2Vec2;
    function GetCoreVertex2: JBox2D.Common.Math.b2Vec2;
    function GetNormalVector: JBox2D.Common.Math.b2Vec2;
    function GetDirectionVector: JBox2D.Common.Math.b2Vec2;
    function GetCorner1Vector: JBox2D.Common.Math.b2Vec2;
    function GetCorner2Vector: JBox2D.Common.Math.b2Vec2;
    function Corner1IsConvex: Boolean;
    function Corner2IsConvex: Boolean;
    function GetFirstVertex(xf: JBox2D.Common.Math.b2Transform): JBox2D.Common.Math.b2Vec2;
    function GetNextEdge: Jb2EdgeShape;
    function GetPrevEdge: Jb2EdgeShape;
    function Support(xf: JBox2D.Common.Math.b2Transform; dX: Float; dY: Float): JBox2D.Common.Math.b2Vec2;
    function RayCast(output: Jb2RayCastOutput; input: Jb2RayCastInput; transform: JBox2D.Common.Math.b2Transform): Boolean;
    function TestPoint(xf: JBox2D.Common.Math.b2Transform; p: JBox2D.Common.Math.b2Vec2): Boolean;
  end;

//Box2D.Collision.Shapes

type
  Jb2MassData = class external 'b2MassData'
    center: JBox2D.Common.Math.b2Vec2;
    I: Float;
    mass: Float;
  end;

//Box2D.Collision.Shapes

type
  Jb2PolygonShape = class external 'b2PolygonShape'(Jb2Shape)
    function AsArray(vertices: array of JBox2D.Common.Math.b2Vec2): Jb2PolygonShape; overload;
    function AsArray(vertices: array of JBox2D.Common.Math.b2Vec2; vertexCount: Float): Jb2PolygonShape; overload;
    function AsBox(hx: Float; hy: Float): Jb2PolygonShape;
    function AsEdge(v1: JBox2D.Common.Math.b2Vec2; b2: JBox2D.Common.Math.b2Vec2): Jb2PolygonShape;
    function AsOrientedBox(hx: Float; hy: Float): Jb2PolygonShape; overload;
    function AsOrientedBox(hx: Float; hy: Float; center: JBox2D.Common.Math.b2Vec2): Jb2PolygonShape; overload;
    function AsOrientedBox(hx: Float; hy: Float; center: JBox2D.Common.Math.b2Vec2; angle: Float): Jb2PolygonShape; overload;
    function AsVector(vertices: array of JBox2D.Common.Math.b2Vec2): Jb2PolygonShape; overload;
    function AsVector(vertices: array of JBox2D.Common.Math.b2Vec2; vertexCount: Float): Jb2PolygonShape; overload;
    procedure ComputeAABB(aabb: Jb2AABB; xf: JBox2D.Common.Math.b2Transform);
    procedure ComputeMass(massData: Jb2MassData; density: Float);
    function ComputeSubmergedArea(normal: JBox2D.Common.Math.b2Vec2; offset: Float; xf: JBox2D.Common.Math.b2Transform; c: JBox2D.Common.Math.b2Vec2): Float;
    function Copy: Jb2PolygonShape;
    function GetNormals: array of JBox2D.Common.Math.b2Vec2;
    function GetSupport(d: JBox2D.Common.Math.b2Vec2): Float;
    function GetSupportVertex(d: JBox2D.Common.Math.b2Vec2): JBox2D.Common.Math.b2Vec2;
    function GetVertexCount: Float;
    function GetVertices: array of JBox2D.Common.Math.b2Vec2;
    function RayCast(output: Jb2RayCastOutput; input: Jb2RayCastInput; transform: JBox2D.Common.Math.b2Transform): Boolean;
    procedure Set(other: Jb2Shape);
    procedure SetAsArray(vertices: array of JBox2D.Common.Math.b2Vec2); overload;
    procedure SetAsArray(vertices: array of JBox2D.Common.Math.b2Vec2; vertexCount: Float); overload;
    procedure SetAsBox(hx: Float; hy: Float);
    procedure SetAsEdge(v1: JBox2D.Common.Math.b2Vec2; b2: JBox2D.Common.Math.b2Vec2);
    procedure SetAsOrientedBox(hx: Float; hy: Float); overload;
    procedure SetAsOrientedBox(hx: Float; hy: Float; center: JBox2D.Common.Math.b2Vec2); overload;
    procedure SetAsOrientedBox(hx: Float; hy: Float; center: JBox2D.Common.Math.b2Vec2; angle: Float); overload;
    procedure SetAsVector(vertices: array of Variant); overload;
    procedure SetAsVector(vertices: array of Variant; vertexCount: Float); overload;
    function TestPoint(xf: JBox2D.Common.Math.b2Transform; p: JBox2D.Common.Math.b2Vec2): Boolean;
  end;

//Box2D.Collision.Shapes

type
  Jb2Shape = class external 'b2Shape'
    e_hitCollide: Float;
    e_missCollide: Float;
    startsInsideCollide: Float;
    e_unknownShape: Float;
    e_circleShape: Float;
    e_polygonShape: Float;
    e_edgeShape: Float;
    e_shapeTypeCount: Float;
    constructor Create;
    procedure ComputeAABB(aabb: Jb2AABB; xf: JBox2D.Common.Math.b2Transform);
    procedure ComputeMass(massData: Jb2MassData; density: Float);
    function ComputeSubmergedArea(normal: JBox2D.Common.Math.b2Vec2; offset: Float; xf: JBox2D.Common.Math.b2Transform; c: JBox2D.Common.Math.b2Vec2): Float;
    function Copy: Jb2Shape;
    function GetType: Float;
    function RayCast(output: Jb2RayCastOutput; input: Jb2RayCastInput; transform: JBox2D.Common.Math.b2Transform): Boolean;
    procedure Set(other: Jb2Shape);
    function TestOverlap(shape1: Jb2Shape; transform1: JBox2D.Common.Math.b2Transform; shape2: Jb2Shape; transform2: JBox2D.Common.Math.b2Transform): Boolean;
    function TestPoint(xf: JBox2D.Common.Math.b2Transform; p: JBox2D.Common.Math.b2Vec2): Boolean;
  end;

//Box2D.Dynamics

type
  Jb2Body = class external 'b2Body'
    b2_dynamicBody: Float;
    b2_kinematicBody: Float;
    b2_staticBody: Float;
    procedure ApplyForce(force: JBox2D.Common.Math.b2Vec2; point: JBox2D.Common.Math.b2Vec2);
    procedure ApplyImpulse(impulse: JBox2D.Common.Math.b2Vec2; point: JBox2D.Common.Math.b2Vec2);
    procedure ApplyTorque(torque: Float);
    function CreateFixture(def: Jb2FixtureDef): Jb2Fixture;
    function CreateFixture2(shape: JBox2D.Collision.Shapes.b2Shape): Jb2Fixture; overload;
    function CreateFixture2(shape: JBox2D.Collision.Shapes.b2Shape; density: Float): Jb2Fixture; overload;
    procedure DestroyFixture(fixture: Jb2Fixture);
    function GetAngle: Float;
    function GetAngularDamping: Float;
    function GetAngularVelocity: Float;
    function GetContactList: JContacts.b2ContactEdge;
    function GetControllerList: JControllers.b2ControllerEdge;
    function GetDefinition: Jb2BodyDef;
    function GetFixtureList: Jb2Fixture;
    function GetInertia: Float;
    function GetJointList: JJoints.b2JointEdge;
    function GetLinearDamping: Float;
    function GetLinearVelocity: JBox2D.Common.Math.b2Vec2;
    function GetLinearVelocityFromLocalPoint(localPoint: JBox2D.Common.Math.b2Vec2): JBox2D.Common.Math.b2Vec2;
    function GetLinearVelocityFromWorldPoint(worldPoint: JBox2D.Common.Math.b2Vec2): JBox2D.Common.Math.b2Vec2;
    function GetLocalCenter: JBox2D.Common.Math.b2Vec2;
    function GetLocalPoint(worldPoint: JBox2D.Common.Math.b2Vec2): JBox2D.Common.Math.b2Vec2;
    function GetLocalVector(worldVector: JBox2D.Common.Math.b2Vec2): JBox2D.Common.Math.b2Vec2;
    function GetMass: Float;
    procedure GetMassData(data: JBox2D.Collision.Shapes.b2MassData);
    function GetNext: Jb2Body;
    function GetPosition: JBox2D.Common.Math.b2Vec2;
    function GetTransform: JBox2D.Common.Math.b2Transform;
    function GetType: Float;
    function GetUserData: Variant;
    function GetWorld: Jb2World;
    function GetWorldCenter: JBox2D.Common.Math.b2Vec2;
    function GetWorldPoint(localPoint: JBox2D.Common.Math.b2Vec2): JBox2D.Common.Math.b2Vec2;
    function GetWorldVector(localVector: JBox2D.Common.Math.b2Vec2): JBox2D.Common.Math.b2Vec2;
    function IsActive: Boolean;
    function IsAwake: Boolean;
    function IsBullet: Boolean;
    function IsFixedRotation: Boolean;
    function IsSleepingAllowed: Boolean;
    procedure Merge(other: Jb2Body);
    procedure ResetMassData;
    procedure SetActive(flag: Boolean);
    procedure SetAngle(angle: Float);
    procedure SetAngularDamping(angularDamping: Float);
    procedure SetAngularVelocity(omega: Float);
    procedure SetAwake(flag: Boolean);
    procedure SetBullet(flag: Boolean);
    procedure SetFixedRotation(fixed: Boolean);
    procedure SetLinearDamping(linearDamping: Float);
    procedure SetLinearVelocity(v: JBox2D.Common.Math.b2Vec2);
    procedure SetMassData(massData: JBox2D.Collision.Shapes.b2MassData);
    procedure SetPosition(position: JBox2D.Common.Math.b2Vec2);
    procedure SetPositionAndAngle(position: JBox2D.Common.Math.b2Vec2; angle: Float);
    procedure SetSleepingAllowed(flag: Boolean);
    procedure SetTransform(xf: JBox2D.Common.Math.b2Transform);
    procedure SetType(&type: Float);
    procedure SetUserData(data: Variant);
    function Split(callback: (fixture: Jb2Fixture): Boolean): Jb2Body;
  end;

//Box2D.Dynamics

type
  Jb2BodyDef = class external 'b2BodyDef'
    active: Boolean;
    allowSleep: Boolean;
    angle: Float;
    angularDamping: Float;
    angularVelocity: Float;
    awake: Boolean;
    bullet: Boolean;
    fixedRotation: Boolean;
    inertiaScale: Float;
    linearDamping: Float;
    linearVelocity: JBox2D.Common.Math.b2Vec2;
    position: JBox2D.Common.Math.b2Vec2;
    &type: Float;
    userData: Variant;
  end;

//Box2D.Dynamics

type
  Jb2ContactFilter = class external 'b2ContactFilter'
    function RayCollide(userData: Variant): Boolean;
    function ShouldCollide(fixtureA: Jb2Fixture; fixtureB: Jb2Fixture): Boolean;
  end;

//Box2D.Dynamics

type
  Jb2ContactImpulse = class external 'b2ContactImpulse'
    normalImpulses: JBox2D.Common.Math.b2Vec2;
    tangentImpulses: JBox2D.Common.Math.b2Vec2;
  end;

//Box2D.Dynamics

type
  Jb2ContactListener = class external 'b2ContactListener'
    procedure BeginContact(contact: JContacts.b2Contact);
    procedure EndContact(contact: JContacts.b2Contact);
    procedure PostSolve(contact: JContacts.b2Contact; impulse: Jb2ContactImpulse);
    procedure PreSolve(contact: JContacts.b2Contact; oldManifold: JBox2D.Collision.b2Manifold);
  end;

//Box2D.Dynamics

type
  Jb2DebugDraw = class external 'b2DebugDraw'
    e_aabbBit: Float;
    e_centerOfMassBit: Float;
    e_controllerBit: Float;
    e_jointBit: Float;
    e_pairBit: Float;
    e_shapeBit: Float;
    constructor Create;
    procedure AppendFlags(flags: Float);
    procedure ClearFlags(flags: Float);
    procedure DrawCircle(center: JBox2D.Common.Math.b2Vec2; radius: Float; color: JBox2D.Common.b2Color);
    procedure DrawPolygon(vertices: array of JBox2D.Common.Math.b2Vec2; vertexCount: Float; color: JBox2D.Common.b2Color);
    procedure DrawSegment(p1: JBox2D.Common.Math.b2Vec2; p2: JBox2D.Common.Math.b2Vec2; color: JBox2D.Common.b2Color);
    procedure DrawSolidCircle(center: JBox2D.Common.Math.b2Vec2; radius: Float; axis: JBox2D.Common.Math.b2Vec2; color: JBox2D.Common.b2Color);
    procedure DrawSolidPolygon(vertices: array of JBox2D.Common.Math.b2Vec2; vertexCount: Float; color: JBox2D.Common.b2Color);
    procedure DrawTransform(xf: JBox2D.Common.Math.b2Transform);
    function GetAlpha: Float;
    function GetDrawScale: Float;
    function GetFillAlpha: Float;
    function GetFlags: Float;
    function GetLineThickness: Float;
    function GetSprite: JCanvasRenderingContext2D;
    function GetXFormScale: Float;
    procedure SetAlpha(alpha: Float);
    procedure SetDrawScale(drawScale: Float);
    procedure SetFillAlpha(alpha: Float);
    procedure SetFlags(flags: Float);
    procedure SetLineThickness(lineThickness: Float);
    procedure SetSprite(canvas: JCanvasRenderingContext2D);
    procedure SetXFormScale(xformScale: Float);
  end;

//Box2D.Dynamics

type
  Jb2DestructionListener = class external 'b2DestructionListener'
    procedure SayGoodbyeFixture(fixture: Jb2Fixture);
    procedure SayGoodbyeJoint(joint: JJoints.b2Joint);
  end;

//Box2D.Dynamics

type
  Jb2FilterData = class external 'b2FilterData'
    categoryBits: Float;
    groupIndex: Float;
    maskBits: Float;
    function Copy: Jb2FilterData;
  end;

//Box2D.Dynamics

type
  Jb2Fixture = class external 'b2Fixture'
    function GetAABB: JBox2D.Collision.b2AABB;
    function GetBody: Jb2Body;
    function GetDensity: Float;
    function GetFilterData: Jb2FilterData;
    function GetFriction: Float;
    function GetMassData: JBox2D.Collision.Shapes.b2MassData; overload;
    function GetMassData(massData: JBox2D.Collision.Shapes.b2MassData): JBox2D.Collision.Shapes.b2MassData; overload;
    function GetNext: Jb2Fixture;
    function GetRestitution: Float;
    function GetShape: JBox2D.Collision.Shapes.b2Shape;
    function GetType: Float;
    function GetUserData: Variant;
    function IsSensor: Boolean;
    function RayCast(output: JBox2D.Collision.b2RayCastOutput; input: JBox2D.Collision.b2RayCastInput): Boolean;
    procedure SetDensity(density: Float);
    procedure SetFilterData(filter: Variant);
    procedure SetFriction(friction: Float);
    procedure SetRestitution(restitution: Float);
    procedure SetSensor(sensor: Boolean);
    procedure SetUserData(data: Variant);
    function TestPoint(p: JBox2D.Common.Math.b2Vec2): Boolean;
  end;

//Box2D.Dynamics

type
  Jb2FixtureDef = class external 'b2FixtureDef'
    density: Float;
    filter: Jb2FilterData;
    friction: Float;
    isSensor: Boolean;
    restitution: Float;
    shape: JBox2D.Collision.Shapes.b2Shape;
    userData: Variant;
    constructor Create;
  end;

//Box2D.Dynamics

type
  Jb2World = class external 'b2World'
    e_locked: Float;
    e_newFixture: Float;
    constructor Create(gravity: JBox2D.Common.Math.b2Vec2; doSleep: Boolean);
    function AddController(c: JControllers.b2Controller): JControllers.b2Controller;
    procedure ClearForces;
    function CreateBody(def: Jb2BodyDef): Jb2Body;
    function CreateController(controller: JControllers.b2Controller): JControllers.b2Controller;
    function CreateJoint(def: JJoints.b2JointDef): JJoints.b2Joint;
    procedure DestroyBody(b: Jb2Body);
    procedure DestroyController(controller: JControllers.b2Controller);
    procedure DestroyJoint(j: JJoints.b2Joint);
    procedure DrawDebugData;
    function GetBodyCount: Float;
    function GetBodyList: Jb2Body;
    function GetContactCount: Float;
    function GetContactList: JContacts.b2Contact;
    function GetGravity: JBox2D.Common.Math.b2Vec2;
    function GetGroundBody: Jb2Body;
    function GetJointCount: Float;
    function GetJointList: JJoints.b2Joint;
    function GetProxyCount: Float;
    function IsLocked: Boolean;
    procedure QueryAABB(callback: (fixutre: Jb2Fixture): Boolean; aabb: JBox2D.Collision.b2AABB);
    procedure QueryPoint(callback: (fixture: Jb2Fixture): Boolean; p: JBox2D.Common.Math.b2Vec2);
    procedure QueryShape(callback: (fixture: Jb2Fixture): Boolean; shape: JBox2D.Collision.Shapes.b2Shape); overload;
    procedure QueryShape(callback: (fixture: Jb2Fixture): Boolean; shape: JBox2D.Collision.Shapes.b2Shape; transform: JBox2D.Common.Math.b2Transform); overload;
    procedure RayCast(callback: (fixture: Jb2Fixture; point: JBox2D.Common.Math.b2Vec2; normal: JBox2D.Common.Math.b2Vec2; fraction: Float): Float; point1: JBox2D.Common.Math.b2Vec2; point2: JBox2D.Common.Math.b2Vec2);
    function RayCastAll(point1: JBox2D.Common.Math.b2Vec2; point2: JBox2D.Common.Math.b2Vec2): array of Jb2Fixture;
    function RayCastOne(point1: JBox2D.Common.Math.b2Vec2; point2: JBox2D.Common.Math.b2Vec2): Jb2Fixture;
    procedure RemoveController(c: JControllers.b2Controller);
    procedure SetBroadPhase(broadPhase: JBox2D.Collision.IBroadPhase);
    procedure SetContactFilter(filter: Jb2ContactFilter);
    procedure SetContactListener(listener: Jb2ContactListener);
    procedure SetContinuousPhysics(flag: Boolean);
    procedure SetDebugDraw(debugDraw: Jb2DebugDraw);
    procedure SetDestructionListener(listener: Jb2DestructionListener);
    procedure SetGravity(gravity: JBox2D.Common.Math.b2Vec2);
    procedure SetWarmStarting(flag: Boolean);
    procedure Step(dt: Float; velocityIterations: Float; positionIterations: Float);
    procedure Validate;
  end;

//Box2D.Dynamics.Contacts

type
  Jb2Contact = class external 'b2Contact'
    constructor Create;
    procedure FlagForFiltering;
    function GetFixtureA: Jb2Fixture;
    function GetFixtureB: Jb2Fixture;
    function GetManifold: JBox2D.Collision.b2Manifold;
    function GetNext: Jb2Contact;
    procedure GetWorldManifold(worldManifold: JBox2D.Collision.b2WorldManifold);
    function IsContinuous: Boolean;
    function IsEnabled: Boolean;
    function IsSensor: Boolean;
    function IsTouching: Boolean;
    procedure SetEnabled(flag: Boolean);
    procedure SetSensor(sensor: Boolean);
  end;

//Box2D.Dynamics.Contacts

type
  Jb2ContactEdge = class external 'b2ContactEdge'
    contact: Jb2Contact;
    next: Jb2ContactEdge;
    other: Jb2Body;
    prev: Jb2ContactEdge;
  end;

//Box2D.Dynamics.Contacts

type
  Jb2ContactResult = class external 'b2ContactResult'
    id: JBox2D.Collision.b2ContactID;
    normal: JBox2D.Common.Math.b2Vec2;
    normalImpulse: Float;
    position: JBox2D.Common.Math.b2Vec2;
    shape1: JBox2D.Collision.Shapes.b2Shape;
    shape2: JBox2D.Collision.Shapes.b2Shape;
    tangentImpulse: Float;
  end;

//Box2D.Dynamics.Controllers

type
  Jb2Controller = class external 'b2Controller'
    m_bodyCount: Float;
    m_bodyList: Jb2ControllerEdge;
    procedure AddBody(body: Jb2Body);
    procedure Clear;
    procedure Draw(debugDraw: Jb2DebugDraw);
    function GetBodyList: Jb2ControllerEdge;
    function GetNext: Jb2Controller;
    function GetWorld: Jb2World;
    procedure RemoveBody(body: Jb2Body);
    procedure Step(step: Variant);
  end;

//Box2D.Dynamics.Controllers

type
  Jb2ControllerEdge = class external 'b2ControllerEdge'
    body: Jb2Body;
    controller: Jb2Controller;
    nextBody: Jb2ControllerEdge;
    nextController: Jb2ControllerEdge;
    prevBody: Jb2ControllerEdge;
    prevController: Jb2ControllerEdge;
  end;

//Box2D.Dynamics.Controllers

type
  Jb2BuoyancyController = class external 'b2BuoyancyController'(Jb2Controller)
    angularDrag: Float;
    density: Float;
    gravity: JBox2D.Common.Math.b2Vec2;
    linearDrag: Float;
    normal: JBox2D.Common.Math.b2Vec2;
    offset: Float;
    useDensity: Boolean;
    useWorldGravity: Boolean;
    velocity: JBox2D.Common.Math.b2Vec2;
  end;

//Box2D.Dynamics.Controllers

type
  Jb2ConstantAccelController = class external 'b2ConstantAccelController'(Jb2Controller)
    A: JBox2D.Common.Math.b2Vec2;
    procedure Step(step: Variant);
  end;

//Box2D.Dynamics.Controllers

type
  Jb2ConstantForceController = class external 'b2ConstantForceController'(Jb2Controller)
    A: JBox2D.Common.Math.b2Vec2;
    procedure Step(step: Variant);
  end;

//Box2D.Dynamics.Controllers

type
  Jb2GravityController = class external 'b2GravityController'(Jb2Controller)
    G: Float;
    invSqr: Boolean;
    procedure Step(step: Variant);
  end;

//Box2D.Dynamics.Controllers

type
  Jb2TensorDampingController = class external 'b2TensorDampingController'(Jb2Controller)
    maxTimeStep: Float;
    T: JBox2D.Common.Math.b2Mat22;
    procedure SetAxisAligned(xDamping: Float; yDamping: Float);
    procedure Step(step: Variant);
  end;

//Box2D.Dynamics.Joints

type
  Jb2Joint = class external 'b2Joint'
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetBodyA: Jb2Body;
    function GetBodyB: Jb2Body;
    function GetNext: Jb2Joint;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
    function GetType: Float;
    function GetUserData: Variant;
    function IsActive: Boolean;
    procedure SetUserData(data: Variant);
  end;

//Box2D.Dynamics.Joints

type
  Jb2JointDef = class external 'b2JointDef'
    bodyA: Jb2Body;
    bodyB: Jb2Body;
    collideConnected: Boolean;
    &type: Float;
    userData: Variant;
    constructor Create;
  end;

//Box2D.Dynamics.Joints

type
  Jb2JointEdge = class external 'b2JointEdge'
    joint: Jb2Joint;
    next: Jb2JointEdge;
    other: Jb2Body;
    prev: Jb2JointEdge;
  end;

//Box2D.Dynamics.Joints

type
  Jb2DistanceJoint = class external 'b2DistanceJoint'(Jb2Joint)
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetDampingRatio: Float;
    function GetFrequency: Float;
    function GetLength: Float;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
    procedure SetDampingRatio(ratio: Float);
    procedure SetFrequency(hz: Float);
    procedure SetLength(_length: Float);
  end;

//Box2D.Dynamics.Joints

type
  Jb2DistanceJointDef = class external 'b2DistanceJointDef'(Jb2JointDef)
    dampingRatio: Float;
    frequencyHz: Float;
    _length: Float;
    localAnchorA: JBox2D.Common.Math.b2Vec2;
    localAnchorB: JBox2D.Common.Math.b2Vec2;
    constructor Create;
    procedure Initialize(bA: Jb2Body; bB: Jb2Body; anchorA: JBox2D.Common.Math.b2Vec2; anchorB: JBox2D.Common.Math.b2Vec2);
  end;

//Box2D.Dynamics.Joints

type
  Jb2FrictionJoint = class external 'b2FrictionJoint'(Jb2Joint)
    m_angularMass: Float;
    m_linearMass: JBox2D.Common.Math.b2Mat22;
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetMaxForce: Float;
    function GetMaxTorque: Float;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
    procedure SetMaxForce(force: Float);
    procedure SetMaxTorque(torque: Float);
  end;

//Box2D.Dynamics.Joints

type
  Jb2FrictionJointDef = class external 'b2FrictionJointDef'(Jb2JointDef)
    localAnchorA: JBox2D.Common.Math.b2Vec2;
    localAnchorB: JBox2D.Common.Math.b2Vec2;
    maxForce: Float;
    maxTorque: Float;
    constructor Create;
    procedure Initialize(bA: Jb2Body; bB: Jb2Body; anchor: JBox2D.Common.Math.b2Vec2);
  end;

//Box2D.Dynamics.Joints

type
  Jb2GearJoint = class external 'b2GearJoint'(Jb2Joint)
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetRatio: Float;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
    procedure SetRatio(ratio: Float);
  end;

//Box2D.Dynamics.Joints

type
  Jb2GearJointDef = class external 'b2GearJointDef'(Jb2JointDef)
    joint1: Jb2Joint;
    joint2: Jb2Joint;
    ratio: Float;
    constructor Create;
  end;

//Box2D.Dynamics.Joints

type
  Jb2LineJoint = class external 'b2LineJoint'(Jb2Joint)
    procedure EnableLimit(flag: Boolean);
    procedure EnableMotor(flag: Boolean);
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetJointSpeed: Float;
    function GetJointTranslation: Float;
    function GetLowerLimit: Float;
    function GetMaxMotorForce: Float;
    function GetMotorForce: Float;
    function GetMotorSpeed: Float;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
    function GetUpperLimit: Float;
    function IsLimitEnabled: Boolean;
    function IsMotorEnabled: Boolean;
    procedure SetLimits(lower: Float; upper: Float);
    procedure SetMaxMotorForce(force: Float);
    procedure SetMotorSpeed(speed: Float);
  end;

//Box2D.Dynamics.Joints

type
  Jb2LineJointDef = class external 'b2LineJointDef'(Jb2JointDef)
    enableLimit: Boolean;
    enableMotor: Boolean;
    localAnchorA: JBox2D.Common.Math.b2Vec2;
    localAnchorB: JBox2D.Common.Math.b2Vec2;
    localAxisA: JBox2D.Common.Math.b2Vec2;
    lowerTranslation: Float;
    maxMotorForce: Float;
    motorSpeed: Float;
    upperTranslation: Float;
    constructor Create;
    procedure Initialize(bA: Jb2Body; bB: Jb2Body; anchor: JBox2D.Common.Math.b2Vec2; axis: JBox2D.Common.Math.b2Vec2);
  end;

//Box2D.Dynamics.Joints

type
  Jb2MouseJoint = class external 'b2MouseJoint'(Jb2Joint)
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetDampingRatio: Float;
    function GetFrequency: Float;
    function GetMaxForce: Float;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
    function GetTarget: JBox2D.Common.Math.b2Vec2;
    procedure SetDampingRatio(ratio: Float);
    procedure SetFrequency(hz: Float);
    procedure SetMaxForce(maxForce: Float);
    procedure SetTarget(target: JBox2D.Common.Math.b2Vec2);
  end;

//Box2D.Dynamics.Joints

type
  Jb2MouseJointDef = class external 'b2MouseJointDef'(Jb2JointDef)
    dampingRatio: Float;
    frequencyHz: Float;
    maxForce: Float;
    constructor Create;
  end;

//Box2D.Dynamics.Joints

type
  Jb2PrismaticJoint = class external 'b2PrismaticJoint'(Jb2Joint)
    procedure EnableLimit(flag: Boolean);
    procedure EnableMotor(flag: Boolean);
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetJointSpeed: Float;
    function GetJointTranslation: Float;
    function GetLowerLimit: Float;
    function GetMotorForce: Float;
    function GetMotorSpeed: Float;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
    function GetUpperLimit: Float;
    function IsLimitEnabled: Boolean;
    function IsMotorEnabled: Boolean;
    procedure SetLimits(lower: Float; upper: Float);
    procedure SetMaxMotorForce(force: Float);
    procedure SetMotorSpeed(speed: Float);
  end;

//Box2D.Dynamics.Joints

type
  Jb2PrismaticJointDef = class external 'b2PrismaticJointDef'(Jb2JointDef)
    enableLimit: Boolean;
    enableMotor: Boolean;
    localAnchorA: JBox2D.Common.Math.b2Vec2;
    localAnchorB: JBox2D.Common.Math.b2Vec2;
    localAxisA: JBox2D.Common.Math.b2Vec2;
    lowerTranslation: Float;
    maxMotorForce: Float;
    motorSpeed: Float;
    referenceAngle: Float;
    upperTranslation: Float;
    constructor Create;
    procedure Initialize(bA: Jb2Body; bB: Jb2Body; anchor: JBox2D.Common.Math.b2Vec2; axis: JBox2D.Common.Math.b2Vec2);
  end;

//Box2D.Dynamics.Joints

type
  Jb2PullyJoint = class external 'b2PullyJoint'(Jb2Joint)
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetGroundAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetGroundAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetLength1: Float;
    function GetLength2: Float;
    function GetRatio: Float;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
  end;

//Box2D.Dynamics.Joints

type
  Jb2PullyJointDef = class external 'b2PullyJointDef'(Jb2JointDef)
    groundAnchorA: JBox2D.Common.Math.b2Vec2;
    groundAnchorB: JBox2D.Common.Math.b2Vec2;
    lengthA: Float;
    lengthB: Float;
    localAnchorA: JBox2D.Common.Math.b2Vec2;
    localAnchorB: JBox2D.Common.Math.b2Vec2;
    maxLengthA: Float;
    maxLengthB: Float;
    ratio: Float;
    constructor Create;
    procedure Initialize(bA: Jb2Body; bB: Jb2Body; gaA: JBox2D.Common.Math.b2Vec2; gaB: JBox2D.Common.Math.b2Vec2; anchorA: JBox2D.Common.Math.b2Vec2; anchorB: JBox2D.Common.Math.b2Vec2);
  end;

//Box2D.Dynamics.Joints

type
  Jb2RevoluteJoint = class external 'b2RevoluteJoint'(Jb2Joint)
    procedure EnableLimit(flag: Boolean);
    procedure EnableMotor(flag: Boolean);
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetJointAngle: Float;
    function GetJointSpeed: Float;
    function GetLowerLimit: Float;
    function GetMotorSpeed: Float;
    function GetMotorTorque: Float;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
    function GetUpperLimit: Float;
    function IsLimitEnabled: Boolean;
    function IsMotorEnabled: Boolean;
    procedure SetLimits(lower: Float; upper: Float);
    procedure SetMaxMotorTorque(torque: Float);
    procedure SetMotorSpeed(speed: Float);
  end;

//Box2D.Dynamics.Joints

type
  Jb2RevoluteJointDef = class external 'b2RevoluteJointDef'(Jb2JointDef)
    enableLimit: Boolean;
    enableMotor: Boolean;
    localAnchorA: JBox2D.Common.Math.b2Vec2;
    localAnchorB: JBox2D.Common.Math.b2Vec2;
    lowerAngle: Float;
    maxMotorTorque: Float;
    motorSpeed: Float;
    referenceAngle: Float;
    upperAngle: Float;
    constructor Create;
    procedure Initialize(bA: Jb2Body; bB: Jb2Body; anchor: JBox2D.Common.Math.b2Vec2);
  end;

//Box2D.Dynamics.Joints

type
  Jb2WeldJoint = class external 'b2WeldJoint'(Jb2Joint)
    function GetAnchorA: JBox2D.Common.Math.b2Vec2;
    function GetAnchorB: JBox2D.Common.Math.b2Vec2;
    function GetReactionForce(inv_dt: Float): JBox2D.Common.Math.b2Vec2;
    function GetReactionTorque(inv_dt: Float): Float;
  end;

//Box2D.Dynamics.Joints

type
  Jb2WeldJointDef = class external 'b2WeldJointDef'(Jb2JointDef)
    localAnchorA: JBox2D.Common.Math.b2Vec2;
    localAnchorB: JBox2D.Common.Math.b2Vec2;
    referenceAngle: Float;
    constructor Create;
    procedure Initialize(bA: Jb2Body; bB: Jb2Body; anchor: JBox2D.Common.Math.b2Vec2);
  end;

