unit Flot;

interface

type
  JplotOptions = class external
    colors: array of Variant; // nullable
    series: JseriesOptions; // nullable
    legend: JlegendOptions; // nullable
    xaxis: JaxisOptions; // nullable
    yaxis: JaxisOptions; // nullable
    xaxes: array of JaxisOptions; // nullable
    yaxes: array of JaxisOptions; // nullable
    grid: JgridOptions; // nullable
    interfaction: Jinteraction; // nullable
    hooks: Jhooks; // nullable
  end;

  Jhooks = class external
    processOptions: array of (plot: Jplot; options: JplotOptions)    end;;
    processRawData: array of (plot: Jplot; series: JdataSeries; data: array of Variant; datapoints: Jdatapoints)    end;;
    processDatapoints: array of (plot: Jplot; series: JdataSeries; datapoints: Jdatapoints)    end;;
    processOffset: array of (plot: Jplot; offset: JcanvasPoint)    end;;
    drawBackground: array of (plot: Jplot; context: JCanvasRenderingContext2D)    end;;
    drawSeries: array of (plot: Jplot; context: JCanvasRenderingContext2D; series: JdataSeries)    end;;
    draw: array of (plot: Jplot; context: JCanvasRenderingContext2D)    end;;
    bindEvents: array of (plot: Jplot; eventHolder: JJQuery)    end;;
    drawOverlay: array of (plot: Jplot; context: JCanvasRenderingContext2D)    end;;
    shutdown: array of (plot: Jplot; eventHolder: JJQuery)    end;;
  end;

  Jinteraction = class external
    redrawOverlayInterval: Float; // nullable
  end;

  JgridOptions = class external
    show: Jbool; // nullable
    aboveData: Jbool; // nullable
    color: Variant; // nullable
    backgroundColor: Variant; // nullable
    margin: Variant; // nullable
    labelMargin: Float; // nullable
    axisMargin: Float; // nullable
    markings: Variant; // nullable
    borderWidth: Float;
    borderColor: Variant; // nullable
    minBorderMargin: Float; // nullable
    clickable: Jbool; // nullable
    hoverable: Jbool; // nullable
    autoHighlight: Jbool; // nullable
    mouseActiveRadius: Float; // nullable
    tickColor: Variant; // nullable
    markingsColor: Variant; // nullable
    markingsLineWidth: Float; // nullable
  end;

  JlegendOptions = class external
    show: Jbool; // nullable
    labelFormatter: (&label: String; series: Variant): String; // nullable
    labelBoxBorderColor: Variant; // nullable
    noColumns: Float; // nullable
    position: String; // nullable
    margin: Variant; // nullable
    backgroundColor: Variant; // nullable
    backgroundOpacity: Float; // nullable
    container: JJQuery; // nullable
    sorted: Variant; // nullable
  end;

  JseriesOptions = class external
    color: Float; // nullable
    &label: String; // nullable
    lines: JlinesOptions; // nullable
    bars: JbarsOptions; // nullable
    points: JpointsOptions; // nullable
    xaxis: Float; // nullable
    yaxis: Float; // nullable
    clickable: Jbool; // nullable
    hoverable: Jbool; // nullable
    shadowSize: Float; // nullable
    highlightColor: Variant; // nullable
  end;

  JdataSeries = class external(JseriesOptions)
    data: array of Variant;
  end;

  JaxisOptions = class external
    show: Jbool; // nullable
    position: String; // nullable
    color: Variant; // nullable
    tickColor: Variant; // nullable
    font: Variant; // nullable
    min: Float; // nullable
    max: Float; // nullable
    autoscaleMargin: Float; // nullable
    transform: (v: Float): Float; // nullable
    inverseTransform: (v: Float): Float; // nullable
    ticks: Variant; // nullable
    tickSize: Variant; // nullable
    minTickSize: Variant; // nullable
    tickFormatter: (t: Float): String; // nullable
    tickDecimals: Float; // nullable
    labelWidth: Float; // nullable
    labelHeight: Float; // nullable
    reserveSpace: Jbool; // nullable
    tickLength: Float; // nullable
    alignTicksWithAxis: Float; // nullable
  end;

  JseriesTypeBase = class external
    show: Jbool; // nullable
    lineWidth: Float; // nullable
    fill: Variant; // nullable
    fillColor: Variant; // nullable
  end;

  JlinesOptions = class external(JseriesTypeBase)
    steps: Jbool; // nullable
  end;

  JbarsOptions = class external(JseriesTypeBase)
    barWidth: Float; // nullable
    align: String; // nullable
    horizontal: Jbool; // nullable
  end;

  JpointsOptions = class external(JseriesTypeBase)
    radius: Float; // nullable
    symbol: Variant; // nullable
  end;

  Jgradient = class external
    colors: array of Variant;
  end;

  Jitem = class external
    datapoint: array of Float;
    dataIndex: Float;
    series: JdataSeries;
    seriesIndex: Float;
    pageX: Float;
    pageY: Float;
  end;

  Jdatapoints = class external
    points: array of Float;
    pointsize: Float;
    format: array of JdatapointFormat;
  end;

  JdatapointFormat = class external
    x: Jbool; // nullable
    y: Jbool; // nullable
    number: Jbool;
    required: Jbool;
    defaultValue: Float; // nullable
  end;

  Jpoint = class external
    x: Float;
    y: Float;
  end;

  JcanvasPoint = class external
    top: Float;
    left: Float;
    bottom: Float; // nullable
    right: Float; // nullable
  end;

  Jaxes = class external
    xaxis: Jaxis;
    yaxis: Jaxis;
    x2axis: Jaxis; // nullable
    y2axis: Jaxis; // nullable
  end;

  Jaxis = class external(JaxisOptions)
    function p2c(point: Variant): JcanvasPoint;
    function c2p(canvasPoint: Variant): Jpoint;
  end;

  Jplot = class external
    procedure highlight(series: JdataSeries; datapoint: Jitem);
    procedure unhightlight;
    procedure unhighlight(series: JdataSeries; datapoint: Jitem);
    procedure setData(data: Variant);
    procedure setupGrid;
    procedure draw;
    procedure triggerRedrawOverlay;
    procedure width;
    procedure height;
    procedure offset;
    procedure pointOffset(point: Jpoint);
    procedure resize;
    procedure shutdown;
    function getData: array of JdataSeries;
    function getAxes: Jaxes;
    function getPlaceholder: JJQuery;
    function getCanvas: JHTMLCanvasElement;
    function getPlotOffset: JcanvasPoint;
    function getOptions: JplotOptions;
  end;

type
  JJQueryStatic = class external
    function plot(placeholder: JJQuery; data: array of Jjquery.flot.dataSeries): Jjquery.flot.plot; overload;
    function plot(placeholder: JJQuery; data: array of Jjquery.flot.dataSeries; options: Jjquery.flot.plotOptions): Jjquery.flot.plot; overload;
    function plot(placeholder: JJQuery; data: array of Variant): Jjquery.flot.plot; overload;
    function plot(placeholder: JJQuery; data: array of Variant; options: Jjquery.flot.plotOptions): Jjquery.flot.plot; overload;
  end;

