unit Matter_js;

interface

//'matter-js'

//Matter

type
  JAxes = class external 'Axes'
    function fromVertices(vertices: JArray{<JVector>}): JArray{<JVector>};
    procedure rotate(axes: JArray{<JVector>}; angle: Float);
  end;

  JBodies = class external 'Bodies'
    function circle(x: Float; y: Float; radius: Float): JBody; overload;
    function circle(x: Float; y: Float; radius: Float; options: JIBodyDefinition): JBody; overload;
    function circle(x: Float; y: Float; radius: Float; options: JIBodyDefinition; maxSides: Float): JBody; overload;
    function polygon(x: Float; y: Float; sides: Float; radius: Float): JBody; overload;
    function polygon(x: Float; y: Float; sides: Float; radius: Float; options: JIBodyDefinition): JBody; overload;
    function rectangle(x: Float; y: Float; width: Float; height: Float): JBody; overload;
    function rectangle(x: Float; y: Float; width: Float; height: Float; options: JIBodyDefinition): JBody; overload;
    function trapezoid(x: Float; y: Float; width: Float; height: Float; slope: Float): JBody; overload;
    function trapezoid(x: Float; y: Float; width: Float; height: Float; slope: Float; options: JIBodyDefinition): JBody; overload;
    function fromVertices(x: Float; y: Float; vertexSets: JArray{<JArray{<JVector>}>}): JBody; overload;
    function fromVertices(x: Float; y: Float; vertexSets: JArray{<JArray{<JVector>}>}; options: JIBodyDefinition): JBody; overload;
    function fromVertices(x: Float; y: Float; vertexSets: JArray{<JArray{<JVector>}>}; options: JIBodyDefinition; flagInternal: Boolean): JBody; overload;
    function fromVertices(x: Float; y: Float; vertexSets: JArray{<JArray{<JVector>}>}; options: JIBodyDefinition; flagInternal: Boolean; removeCollinear: Float): JBody; overload;
    function fromVertices(x: Float; y: Float; vertexSets: JArray{<JArray{<JVector>}>}; options: JIBodyDefinition; flagInternal: Boolean; removeCollinear: Float; minimumArea: Float): JBody; overload;
  end;

  JBody = class external 'Body'
    procedure applyForce(body: JBody; position: JVector; force: JVector);
    function create(options: JIBodyDefinition): JBody;
    procedure rotate(body: JBody; rotation: Float);
    function nextGroup(isNonColliding: Boolean): Float;
    function nextCategory: Float;
    procedure set(body: JBody; settings: Variant); overload;
    procedure set(body: JBody; settings: Variant; value: Variant); overload;
    procedure setMass(body: JBody; mass: Float);
    procedure setDensity(body: JBody; density: Float);
    procedure setInterna(body: JBody; interna: Float);
    procedure setVertices(body: JBody; vertices: JArray{<JVector>});
    procedure setParts(body: JBody; parts: JBody; autoHull: Boolean);
    procedure setPosition(body: JBody; position: JVector);
    procedure setAngle(body: JBody; angle: Float);
    procedure setVelocity(body: JBody; velocity: JVector);
    procedure setAngularVelocity(body: JBody; velocity: Float);
    procedure setStatic(body: JBody; isStatic: Boolean);
    procedure scale(body: JBody; scaleX: Float; scaleY: Float); overload;
    procedure scale(body: JBody; scaleX: Float; scaleY: Float; point: JVector); overload;
    procedure translate(body: JBody; translation: JVector);
    procedure update(body: JBody; deltaTime: Float; timeScale: Float; correction: Float);
    angle: Float;
    angularSpeed: Float;
    angularVelocity: Float;
    area: Float;
    axes: JArray{<JVector>};
    bounds: JBounds;
    density: Float;
    force: JVector;
    friction: Float;
    frictionAir: Float;
    id: Float;
    inertia: Float;
    inverseInertia: Float;
    inverseMass: Float;
    isSleeping: Boolean;
    isStatic: Boolean;
    &label: String;
    mass: Float;
    motion: Float;
    position: JVector;
    render: JIBodyRenderOptions;
    restitution: Float;
    sleepThreshold: Float;
    slop: Float;
    speed: Float;
    timeScale: Float;
    torque: Float;
    &type: String;
    velocity: JVector;
    vertices: JArray{<JVector>};
    parts: JArray{<JBody>};
    parent: JBody;
    frictionStatic: Float;
    collisionFilter: JICollisionFilter;
  end;

  JBounds = class external 'Bounds'
  end;

  JComposite = class external 'Composite'
    function add(composite: JComposite; &object: Variant {JBody or JComposite or JConstraint}): JComposite;
    function allBodies(composite: JComposite): JArray{<JBody>};
    function allComposites(composite: JComposite): JArray{<JComposite>};
    function allConstraints(composite: JComposite): JArray{<JComposite>};
    procedure clear(composite: JComposite; keepStatic: Boolean); overload;
    procedure clear(composite: JComposite; keepStatic: Boolean; deep: Boolean); overload;
    function create: JComposite; overload;
    function create(options: JICompositeDefinition): JComposite; overload;
    function get(composite: JComposite; id: Float; &type: String): Variant {JBody or JComposite or JConstraint};
    function move(compositeA: JComposite; objects: JArray{<Variant {JBody or JComposite or JConstraint}>}; compositeB: JComposite): JComposite;
    function rebase(composite: JComposite): JComposite;
    function remove(composite: JComposite; &object: Variant {JBody or JComposite or JConstraint}): JComposite; overload;
    function remove(composite: JComposite; &object: Variant {JBody or JComposite or JConstraint}; deep: Boolean): JComposite; overload;
    procedure setModified(composite: JComposite; isModified: Boolean); overload;
    procedure setModified(composite: JComposite; isModified: Boolean; updateParents: Boolean); overload;
    procedure setModified(composite: JComposite; isModified: Boolean; updateParents: Boolean; updateChildren: Boolean); overload;
    procedure translate(composite: JComposite; translation: JVector); overload;
    procedure translate(composite: JComposite; translation: JVector; recursive: Boolean); overload;
    procedure rotate(composite: JComposite; rotation: Float; point: JVector); overload;
    procedure rotate(composite: JComposite; rotation: Float; point: JVector; recursive: Boolean); overload;
    procedure scale(composite: JComposite; scaleX: Float; scaleY: Float; point: JVector); overload;
    procedure scale(composite: JComposite; scaleX: Float; scaleY: Float; point: JVector; recursive: Boolean); overload;
    bodies: JArray{<JBody>};
    composites: JArray{<JComposite>};
    constraints: JArray{<JConstraint>};
    id: Float;
    isModified: Boolean;
    &label: String;
    parent: JComposite;
    &type: JString;
  end;

  JComposites = class external 'Composites'
    function car(xx: Float; yy: Float; width: Float; height: Float; wheelSize: Float): JComposite;
    function chain(composite: JComposite; xOffsetA: Float; yOffsetA: Float; xOffsetB: Float; yOffsetB: Float; options: Variant): JComposite;
    function mesh(composite: JComposite; columns: Float; rows: Float; crossBrace: Boolean; options: Variant): JComposite;
    function newtonsCradle(xx: Float; yy: Float; _number: Float; size: Float; _length: Float): JComposite;
    function pyramid(xx: Float; yy: Float; columns: Float; rows: Float; columnGap: Float; rowGap: Float; callback: JFunction): JComposite;
    function softBody(xx: Float; yy: Float; columns: Float; rows: Float; columnGap: Float; rowGap: Float; crossBrace: Boolean; particleRadius: Float; particleOptions: Variant; constraintOptions: Variant): JComposite;
    function stack(xx: Float; yy: Float; columns: Float; rows: Float; columnGap: Float; rowGap: Float; callback: JFunction): JComposite;
  end;

  JConstraint = class external 'Constraint'
    function create(options: JIConstraintDefinition): JConstraint;
    bodyA: JBody;
    bodyB: JBody;
    id: Float;
    &label: String;
    _length: Float;
    pointA: JVector;
    pointB: JVector;
    render: JIConstraintRenderDefinition;
    stiffness: Float;
    &type: String;
  end;

  JEngine = class external 'Engine'
    procedure clear(engine: JEngine);
    function create: JEngine; overload;
    function create(element: Variant {JHTMLElement or JIEngineDefinition}): JEngine; overload;
    function create(element: Variant {JHTMLElement or JIEngineDefinition}; options: JIEngineDefinition): JEngine; overload;
    procedure merge(engineA: JEngine; engineB: JEngine);
    function update(engine: JEngine; delta: Float): JEngine; overload;
    function update(engine: JEngine; delta: Float; correction: Float): JEngine; overload;
    procedure run(enige: JEngine);
    broadphase: JGrid;
    constraintIterations: Float;
    enabled: Boolean;
    enableSleeping: Boolean;
    positionIterations: Float;
    render: JRender;
    timing: JIEngineTimingOptions;
    velocityIterations: Float;
    world: JWorld;
  end;

  JGrid = class external 'Grid'
    function create: JGrid; overload;
    function create(options: JIGridDefinition): JGrid; overload;
    procedure update(grid: JGrid; bodies: JArray{<JBody>}; engine: JEngine; forceUpdate: Boolean);
    procedure clear(grid: JGrid);
  end;

  JMouseConstraint = class external 'MouseConstraint'
    function create(engine: JEngine; options: JIMouseConstraintDefinition): JMouseConstraint;
    constraint: JConstraint;
    collisionFilter: JICollisionFilter;
    body: JBody;
    mouse: JMouse;
    &type: String;
  end;

  JQuery = class external 'Query'
    function ray(bodies: JArray{<JBody>}; startPoint: JVector; endPoint: JVector): JArray{<Variant>}; overload;
    function ray(bodies: JArray{<JBody>}; startPoint: JVector; endPoint: JVector; rayWidth: Float): JArray{<Variant>}; overload;
    function region(bodies: JArray{<JBody>}; bounds: JBounds): JArray{<JBody>}; overload;
    function region(bodies: JArray{<JBody>}; bounds: JBounds; outside: Boolean): JArray{<JBody>}; overload;
    function point(bodies: JArray{<JBody>}; point: JVector): JArray{<JBody>};
  end;

  JRender = class external 'Render'
    function create(options: JIRenderDefinition): JRender;
    procedure setPixelRatio(render: JRender; pixelRatio: Float);
    procedure world(engine: JEngine);
    controller: Variant;
    element: JHTMLElement;
    canvas: JHTMLCanvasElement;
    options: JIRendererOptions;
    bounds: JBounds;
    context: JCanvasRenderingContext2D;
    textures: Variant;
  end;

  JRunner = class external 'Runner'
    function create(options: JIRunnerOptions): JRunner;
    function run(runner: JRunner; engine: JEngine): JRunner;
    function run(engine: JEngine): JRunner;
    procedure tick(runner: JRunner; engine: JEngine; time: Float);
    procedure stop(runner: JRunner);
    procedure start(runner: JRunner; engine: JEngine);
    enabled: Boolean;
    isFixed: Boolean;
    delta: Float;
  end;

  JSleeping = class external 'Sleeping'
    procedure set(body: JBody; isSleeping: Boolean);
  end;

  JSvg = class external 'Svg'
    function pathToVertices(path: JSVGPathElement; sampleLength: Float): JArray{<JVector>};
  end;

  JVector = class external 'Vector'
    x: Float;
    y: Float;
    function create: JVector; overload;
    function create(x: Float): JVector; overload;
    function create(x: Float; y: Float): JVector; overload;
    function clone(vector: JVector): JVector;
    function cross3(vectorA: JVector; vectorB: JVector; vectorC: JVector): Float;
    function add(vectorA: JVector; vectorB: JVector): JVector; overload;
    function add(vectorA: JVector; vectorB: JVector; output: JVector): JVector; overload;
    function angle(vectorA: JVector; vectorB: JVector): Float;
    function cross(vectorA: JVector; vectorB: JVector): Float;
    function div(vector: JVector; scalar: Float): JVector;
    function dot(vectorA: JVector; vectorB: JVector): JNumber;
    function magnitude(vector: JVector): Float;
    function magnitudeSquared(vector: JVector): Float;
    function mult(vector: JVector; scalar: Float): JVector;
    function neg(vector: JVector): JVector;
    function normalise(vector: JVector): JVector;
    function perp(vector: JVector): JVector; overload;
    function perp(vector: JVector; negate: Boolean): JVector; overload;
    function rotate(vector: JVector; angle: Float): JVector;
    function rotateAbout(vector: JVector; angle: Float; point: JVector): JVector; overload;
    function rotateAbout(vector: JVector; angle: Float; point: JVector; output: JVector): JVector; overload;
    function sub(vectorA: JVector; vectorB: JVector): JVector; overload;
    function sub(vectorA: JVector; vectorB: JVector; optional: JVector): JVector; overload;
  end;

  JVertices = class external 'Vertices'
    function mean(vertices: JArray{<JVector>}): JArray{<JVector>};
    function clockwiseSort(vertices: JArray{<JVector>}): JArray{<JVector>};
    function isConvex(vertices: JArray{<JVector>}): Boolean;
    function hull(vertices: JArray{<JVector>}): JArray{<JVector>};
    function area(vertices: JArray{<JVector>}; signed: Boolean): Float;
    function centre(vertices: JArray{<JVector>}): JVector;
    procedure chamfer(vertices: JArray{<JVector>}; radius: JArray{<Float>}; quality: Float; qualityMin: Float; qualityMax: Float);
    function contains(vertices: JArray{<JVector>}; point: JVector): Boolean;
    procedure create(points: JArray{<JVector>}; body: JBody);
    function fromPath(path: String; body: JBody): JArray{<JVector>};
    function inertia(vertices: JArray{<JVector>}; mass: Float): Float;
    procedure rotate(vertices: JArray{<JVector>}; angle: Float; point: JVector);
    procedure scale(vertices: JArray{<JVector>}; scaleX: Float; scaleY: Float; point: JVector);
    procedure translate(vertices: JArray{<JVector>}; vector: JVector; scalar: Float);
  end;

  JWorld = class external 'World'
    function add(world: JWorld; body: Variant {JBody or JArray{<JBody>} or JComposite or JArray{<JComposite>} or JConstraint or JArray{<JConstraint>}}): JWorld;
    function addBody(world: JWorld; body: JBody): JWorld;
    function addComposite(world: JWorld; composite: JComposite): JWorld;
    function addConstraint(world: JWorld; constraint: JConstraint): JWorld;
    procedure clear(world: JWorld; keepStatic: Boolean);
    function create(options: JIWorldDefinition): JWorld;
    gravity: JVector;
    bounds: JBounds;
  end;

  JMouse = class external 'Mouse'
    function create(element: JHTMLElement): JMouse;
    procedure setElement(mouse: JMouse; element: JHTMLElement);
    procedure clearSourceEvents(mouse: JMouse);
    procedure setOffset(mouse: JMouse; offset: JVector);
    procedure setScale(mouse: JMouse; scale: JVector);
    element: JHTMLElement;
    absolute: JIMousePoint;
    position: JIMousePoint;
    mousedownPosition: JIMousePoint;
    mouseupPosition: JIMousePoint;
    offset: JIMousePoint;
    scale: JIMousePoint;
    wheelDelta: Float;
    button: Float;
    pixelRatio: Float;
  end;

  JEvents = class external 'Events'
    procedure on(obj: JEngine; name: JsleepStart; callback: (e: JIEvent{<JBody>}));
    procedure on(obj: JEngine; name: JsleepEnd; callback: (e: JIEvent{<JBody>}));
    procedure on(obj: JEngine; name: JbeforeAdd; callback: (e: JIEventComposite{<JComposite>}));
    procedure on(obj: JEngine; name: JafterAdd; callback: (e: JIEventComposite{<JComposite>}));
    procedure on(obj: JEngine; name: JbeforeRemove; callback: (e: JIEventComposite{<JComposite>}));
    procedure on(obj: JEngine; name: JafterRemove; callback: (e: JIEventComposite{<JComposite>}));
    procedure on(obj: JEngine; name: JafterUpdate; callback: (e: JIEventTimestamped{<JEngine>}));
    procedure on(obj: JEngine; name: JbeforeRender; callback: (e: JIEventTimestamped{<JRender>}));
    procedure on(obj: JEngine; name: JafterRender; callback: (e: JIEventTimestamped{<JRender>}));
    procedure on(obj: JEngine; name: JbeforeUpdate; callback: (e: JIEventTimestamped{<JEngine>}));
    procedure on(obj: JEngine; name: JcollisionActive; callback: (e: JIEventCollision{<JEngine>}));
    procedure on(obj: JEngine; name: JcollisionEnd; callback: (e: JIEventCollision{<JEngine>}));
    procedure on(obj: JEngine; name: JcollisionStart; callback: (e: JIEventCollision{<JEngine>}));
    procedure on(obj: JEngine; name: JbeforeTick; callback: (e: JIEventTimestamped{<JRunner>}));
    procedure on(obj: JEngine; name: Jtick; callback: (e: JIEventTimestamped{<JRunner>}));
    procedure on(obj: JEngine; name: JafterTick; callback: (e: JIEventTimestamped{<JRunner>}));
    procedure on(obj: JEngine; name: JbeforeRender; callback: (e: JIEventTimestamped{<JRunner>}));
    procedure on(obj: JEngine; name: JafterRender; callback: (e: JIEventTimestamped{<JRunner>}));
    procedure on(obj: JEngine; name: Jmousedown; callback: (e: Variant));
    procedure on(obj: JEngine; name: Jmousemove; callback: (e: Variant));
    procedure on(obj: JEngine; name: Jmouseup; callback: (e: Variant));
    procedure on(obj: JEngine; name: String; callback: (e: Variant));
    procedure off(obj: Variant; eventName: String; callback: (e: Variant));
    procedure trigger(&object: Variant; eventNames: String; event: (e: Variant));
  end;

  JIBodyDefinition = class external
    angle: Float; // nullable
    angularSpeed: Float; // nullable
    angularVelocity: Float; // nullable
    area: Float; // nullable
    axes: JArray{<JVector>}; // nullable
    bounds: JBounds; // nullable
    density: Float; // nullable
    force: JVector; // nullable
    friction: Float; // nullable
    frictionAir: Float; // nullable
    id: Float; // nullable
    inertia: Float; // nullable
    inverseInertia: Float; // nullable
    inverseMass: Float; // nullable
    isSleeping: Boolean; // nullable
    isStatic: Boolean; // nullable
    &label: String; // nullable
    mass: Float; // nullable
    motion: Float; // nullable
    position: JVector; // nullable
    render: JIBodyRenderOptions; // nullable
    restitution: Float; // nullable
    sleepThreshold: Float; // nullable
    slop: Float; // nullable
    speed: Float; // nullable
    timeScale: Float; // nullable
    torque: Float; // nullable
    &type: String; // nullable
    velocity: JVector; // nullable
    vertices: JArray{<JVector>}; // nullable
    parts: JArray{<JBody>}; // nullable
    parent: JBody; // nullable
    frictionStatic: Float; // nullable
    collisionFilter: JICollisionFilter; // nullable
  end;

  JIBodyRenderOptions = class external
    visible: Boolean;
    sprite: JIBodyRenderOptionsSprite;
    fillStyle: String;
    lineWidth: Float;
    strokeStyle: String;
  end;

  JIBodyRenderOptionsSprite = class external
    texture: String;
    xScale: Float;
    yScale: Float;
  end;

  JIBound = class external
    min:       x: Float;
      y: Float;
    end;;
    max:       x: Float;
      y: Float;
    end;;
  end;

  JICompositeDefinition = class external
    bodies: JArray{<JBody>}; // nullable
    composites: JArray{<JComposite>}; // nullable
    constraints: JArray{<JConstraint>}; // nullable
    id: Float; // nullable
    isModified: Boolean; // nullable
    &label: String; // nullable
    parent: JComposite; // nullable
    &type: JString; // nullable
  end;

  JIConstraintDefinition = class external
    bodyA: JBody; // nullable
    bodyB: JBody; // nullable
    id: Float; // nullable
    &label: String; // nullable
    _length: Float; // nullable
    pointA: JVector; // nullable
    pointB: JVector; // nullable
    render: JIConstraintRenderDefinition; // nullable
    stiffness: Float; // nullable
    &type: String; // nullable
  end;

  JIConstraintRenderDefinition = class external
    lineWidth: Float;
    strokeStyle: String;
    visible: Boolean;
  end;

  JIEngineDefinition = class external
    positionIterations: Float; // nullable
    velocityIterations: Float; // nullable
    constraintIterations: Float; // nullable
    enableSleeping: Boolean; // nullable
    timing: JIEngineTimingOptions; // nullable
    grid: JGrid; // nullable
    world: JWorld; // nullable
  end;

  JIEngineTimingOptions = class external
    timeScale: Float;
    timestamp: Float;
  end;

  JIGridDefinition = class external
  end;

  JIMouseConstraintDefinition = class external
    constraint: JConstraint; // nullable
    collisionFilter: JICollisionFilter; // nullable
    body: JBody; // nullable
    mouse: JMouse; // nullable
    &type: String; // nullable
  end;

  JIPair = class external
    id: Float;
    bodyA: JBody;
    bodyB: JBody;
    contacts: Variant;
    activeContacts: Variant;
    separation: Float;
    isActive: Boolean;
    timeCreated: Float;
    timeUpdated: Float;
    inverseMass: Float;
    friction: Float;
    frictionStatic: Float;
    restitution: Float;
    slop: Float;
  end;

  JIRenderDefinition = class external
    controller: Variant; // nullable
    element: JHTMLElement; // nullable
    canvas: JHTMLCanvasElement; // nullable
    options: JIRendererOptions; // nullable
    bounds: JBounds; // nullable
    context: JCanvasRenderingContext2D; // nullable
    textures: Variant; // nullable
  end;

  JIRendererOptions = class external
    width: Float; // nullable
    height: Float; // nullable
    hasBounds: Boolean; // nullable
  end;

  JIRunnerOptions = class external
    isFixed: Boolean; // nullable
    delta: Float; // nullable
  end;

  JIWorldDefinition = class external(JICompositeDefinition)
    gravity: JVector; // nullable
    bounds: JBounds; // nullable
  end;

  JICollisionFilter = class external
    category: Float;
    mask: Float;
    group: Float;
  end;

  JIMousePoint = class external
    x: Float;
    y: Float;
  end;

  JIEvent{<T>} = class external
    name: String;
    source: JT;
  end;

  JIEventComposite{<T>} = class external(JIEvent {JT} )
    &object: Variant;
  end;

  JIEventTimestamped{<T>} = class external(JIEvent {JT} )
    timestamp: Float;
  end;

  JIEventCollision{<T>} = class external(JIEventTimestamped {JT} )
    pairs: JArray{<JIPair>};
  end;

